package net.sys42.dakedres.grafting;

import com.mojang.serialization.Codec;
import eu.pb4.polymer.core.api.block.PolymerBlockUtils;
import eu.pb4.polymer.core.api.other.PolymerComponent;
import eu.pb4.polymer.resourcepack.api.PolymerResourcePackUtils;
import net.fabricmc.api.ModInitializer;

import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.event.registry.DynamicRegistries;
import net.fabricmc.fabric.api.event.registry.FabricRegistryBuilder;
import net.fabricmc.fabric.api.event.registry.RegistryAttribute;
import net.fabricmc.fabric.api.object.builder.v1.block.entity.FabricBlockEntityTypeBuilder;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.Blocks;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.component.ComponentType;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.registry.*;
import net.minecraft.resource.Resource;
import net.minecraft.sound.BlockSoundGroup;
import net.minecraft.util.Identifier;
import net.sys42.dakedres.grafting.block.Workbench;
import net.sys42.dakedres.grafting.block.entity.WorkbenchBlockEntity;
import net.sys42.dakedres.grafting.data.PlacementOrder;
import net.sys42.dakedres.grafting.data.PlacementOrderReloadListener;
import net.sys42.dakedres.grafting.item.WorkbenchItem;
import net.sys42.dakedres.grafting.ui.Icons;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.stream.IntStream;

public class Grafting implements ModInitializer {
	public static final String MOD_ID = "grafting";
	public static final Identifier WORKBENCH_ID = Identifier.of(Grafting.MOD_ID, "workbench");
	public static final Identifier PLACEMENT_ORDER_COMPONENT_ID = Identifier.of(Grafting.MOD_ID, "placement_order");

	// This logger is used to write text to the console and the log file.
	// It is considered best practice to use your mod id as the logger's name.
	// That way, it's clear which mod wrote info, warnings, and errors.
	public static final Logger LOGGER = LoggerFactory.getLogger(MOD_ID);

	public static final Block WORKBENCH = Registry.register(
			Registries.BLOCK, WORKBENCH_ID,
			new Workbench(
					// TODO: A bit easier to break?
					Block.Settings.copy(Blocks.CRAFTING_TABLE)
			)
	);
	public static final BlockEntityType<WorkbenchBlockEntity> WORKBENCH_BLOCK_ENTITY_TYPE = Registry.register(
			Registries.BLOCK_ENTITY_TYPE,
			WORKBENCH_ID,
			FabricBlockEntityTypeBuilder.create(WorkbenchBlockEntity::new, WORKBENCH).build()
	);
	public static final BlockItem WORKBENCH_ITEM = Registry.register(
			Registries.ITEM, WORKBENCH_ID,
			new WorkbenchItem(WORKBENCH, new Item.Settings())
	);

	public static final ComponentType<Identifier> PLACEMENT_ORDER_COMPONENT_TYPE = Registry.register(
			Registries.DATA_COMPONENT_TYPE,
			PLACEMENT_ORDER_COMPONENT_ID,
			ComponentType.<Identifier>builder().codec(Identifier.CODEC).build()
	);

	public static final RegistryKey<Registry<PlacementOrder>> PLACEMENT_ORDER_REGISTRY_KEY = RegistryKey.ofRegistry(Identifier.of(Grafting.MOD_ID, "placement_order"));
//	public static final Registry<PlacementOrder> PLACEMENT_ORDER_REGISTRY = FabricRegistryBuilder.createSimple(PLACEMENT_ORDER_REGISTRY_KEY)
//			.attribute(RegistryAttribute.MODDED)
//			.buildAndRegister();

	@Override
	public void onInitialize() {
		// This code runs as soon as Minecraft is in a mod-load-ready state.
		// However, some things (like resources) may still be uninitialized.
		// Proceed with mild caution.

		LOGGER.info("Hello Fabric world!");

		PolymerBlockUtils.registerBlockEntity(WORKBENCH_BLOCK_ENTITY_TYPE);
		PolymerComponent.registerDataComponent(PLACEMENT_ORDER_COMPONENT_TYPE);
		Icons.init(); // Static initialization
//		PlacementOrderReloadListener.init();

		DynamicRegistries.register(PLACEMENT_ORDER_REGISTRY_KEY, PlacementOrder.CODEC);

		if (PolymerResourcePackUtils.addModAssets(MOD_ID)) {
			LOGGER.info("Successfully added mod assets for " + MOD_ID);
		} else {
			LOGGER.error("Failed to add mod assets for " + MOD_ID);
		}
		PolymerResourcePackUtils.markAsRequired();
	}
}