package net.sys42.dakedres.grafting;

import net.minecraft.item.ItemStack;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.Identifier;
import net.sys42.dakedres.grafting.data.PlacementOrder;

public class PlacementUtils {
	public static final int[][] ROTATION_MATRICES = {
			{ 0,1,2,3,4,5,6,7,8 },
			{ 1,2,5,0,4,8,3,6,7 },
			{ 2,5,8,1,4,7,0,3,6 },
			{ 5,8,7,2,4,6,1,0,3 },
			{ 8,7,6,5,4,3,2,1,0 },
			{ 7,6,3,8,4,0,5,2,1 },
			{ 6,3,0,7,4,1,8,5,2 },
			{ 3,0,1,6,4,2,7,8,5 }
	};


//	public static DefaultedList<ItemStack> rotate(DefaultedList<ItemStack> grid, int direction) {
//		DefaultedList<ItemStack> o = DefaultedList.ofSize(grid.size(), ItemStack.EMPTY);
//		int[] rotationMatrix = ROTATION_MATRICES[(direction < 0 ? 8 - direction : direction) % 8];
//
//		for(int i = 0; i < grid.size(); i++) {
//			o.set(rotationMatrix[i], grid.get(i));
//		}
//
//		return o;
//	}

	public static int clampRotation(int rotation) {
		return (rotation < 0 ? 8 + rotation : rotation) % 8;
	}

	public static int[] getRotationMatrix(int rotation) {
		return ROTATION_MATRICES[clampRotation(rotation)];
	}

	public static int[] rotate(int[] matrix, int rotation) {
		if(rotation == 0) {
			return matrix;
		}

		int[] o = new int[matrix.length];
		int[] rotationMatrix = ROTATION_MATRICES[rotation];

		for(int i = 0; i < matrix.length; i++) {
			o[rotationMatrix[i]] = matrix[i];
		}

		return o;
	}

	public static int[] getPlacements(ItemStack itemStack, ServerWorld world) {
//		int[] placements = {
//				2,0,8,
//				1,3,4,
//				7,6,5
//		};
//		return placements;


		return getPlacementOrder(itemStack, world).getOrderMatrix();
	}

	public static PlacementOrder getPlacementOrder(ItemStack itemStack, ServerWorld world) {
		Identifier placementIdentifier = itemStack.get(Grafting.PLACEMENT_ORDER_COMPONENT_TYPE);

		if(placementIdentifier == null) {
			placementIdentifier = PlacementOrder.DEFAULT_ID;
		}

		PlacementOrder placementOrder = world.getRegistryManager().get(Grafting.PLACEMENT_ORDER_REGISTRY_KEY).get(placementIdentifier);
		if(placementOrder == null) {
			return PlacementOrder.DEFAULT_FALLBACK;
		}

		return placementOrder;
	};

	public static int[] asIndexes(int[] placements) {
		int[] out = new int[9];

		for(int i = 0; i < 9; i++) {
			out[placements[i]] = i;
		}

		return out;
	}
}
