package net.sys42.dakedres.grafting.block;

import com.mojang.serialization.MapCodec;
import eu.pb4.polymer.blocks.api.BlockModelType;
import eu.pb4.polymer.blocks.api.PolymerBlockModel;
import eu.pb4.polymer.blocks.api.PolymerBlockResourceUtils;
import eu.pb4.polymer.blocks.api.PolymerTexturedBlock;
import eu.pb4.polymer.core.api.block.PolymerBlock;
import net.minecraft.block.*;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.fluid.FluidState;
import net.minecraft.fluid.Fluids;

import net.minecraft.item.ItemPlacementContext;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.Property;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Identifier;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.WorldAccess;
import net.sys42.dakedres.grafting.Grafting;
import net.sys42.dakedres.grafting.block.entity.WorkbenchBlockEntity;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import static net.minecraft.state.property.Properties.WATERLOGGED;

// TODO: Loot tables, including dropping contents when broken
public class Workbench extends BlockWithEntity implements PolymerBlock, Waterloggable, PolymerTexturedBlock {
	private final BlockState waterloggedModel;
	private final BlockState model;

	public Workbench(Settings settings) {
		super(settings);
		this.setDefaultState(this.getDefaultState().with(WATERLOGGED, false));

		var model = PolymerBlockModel.of(Identifier.of(Grafting.MOD_ID, "block/workbench"));
		this.model = PolymerBlockResourceUtils.requestBlock(BlockModelType.TRANSPARENT_BLOCK, model);
		waterloggedModel = PolymerBlockResourceUtils.requestBlock(BlockModelType.TRANSPARENT_BLOCK_WATERLOGGED, model);
	}

	@Override
	protected MapCodec<? extends BlockWithEntity> getCodec() {
		return createCodec(Workbench::new);
	}

	@Override
	public @Nullable BlockEntity createBlockEntity(BlockPos pos, BlockState state) {
		return new WorkbenchBlockEntity(pos, state);
	}

	@Override
	public BlockState getPolymerBlockState(BlockState state) {
		return state.get(WATERLOGGED) ? waterloggedModel : model;
	}

	// Just so it plays the right break sound
	@Override
	public BlockState getPolymerBreakEventBlockState(BlockState state, ServerPlayerEntity player) {
		return Blocks.CRAFTING_TABLE.getDefaultState();
	}

	@Nullable
	public BlockState getPlacementState(@NotNull ItemPlacementContext ctx) {
		WorldAccess worldAccess = ctx.getWorld();
		BlockPos blockPos = ctx.getBlockPos();
		return this.getDefaultState().with(WATERLOGGED, worldAccess.getFluidState(blockPos).getFluid() == Fluids.WATER);
	}

	protected FluidState getFluidState(BlockState state) {
		return state.get(WATERLOGGED) ? Fluids.WATER.getStill(false) : super.getFluidState(state);
	}

	protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
		builder.add(new Property[]{WATERLOGGED});
	}

	@Override
	protected ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, BlockHitResult hit) {
		if(!player.isSneaking() && world.getBlockEntity(pos) instanceof WorkbenchBlockEntity blockEntity) {
//			((ServerPlayerEntity) player).openHandledScreen(blockEntity);
			blockEntity.openMenu((ServerPlayerEntity) player);
			return ActionResult.SUCCESS;
		}

		return super.onUse(state, world, pos, player, hit);
	}
}
