package net.sys42.dakedres.grafting.block.entity;

import eu.pb4.polymer.resourcepack.api.PolymerModelData;
import eu.pb4.sgui.api.ClickType;
import eu.pb4.sgui.api.elements.GuiElement;
import eu.pb4.sgui.virtual.inventory.VirtualSlot;
import net.minecraft.block.BlockState;
import net.minecraft.block.CrafterBlock;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.inventory.CraftingResultInventory;
import net.minecraft.inventory.Inventory;
import net.minecraft.inventory.RecipeInputInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NbtCompound;
import net.minecraft.recipe.RecipeMatcher;
import net.minecraft.recipe.input.CraftingRecipeInput;
import net.minecraft.registry.RegistryWrapper;
import net.minecraft.screen.ScreenHandlerType;
import net.minecraft.screen.slot.CraftingResultSlot;
import net.minecraft.screen.slot.Slot;
import net.minecraft.screen.slot.SlotActionType;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.text.Style;
import net.minecraft.text.Text;
import net.minecraft.util.Formatting;
import net.minecraft.util.Identifier;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.sys42.dakedres.grafting.Grafting;
import net.sys42.dakedres.grafting.PlacementUtils;
import net.sys42.dakedres.grafting.ui.CursorListeningSimpleGui;
import net.sys42.dakedres.grafting.ui.Icons;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

public class WorkbenchBlockEntity extends OrderedContainer {
	public static final String NBT_ORIENTATION_KEY = "CraftingOrientation";
	public static final int GRID_AREA = 9;
	public static final int GRID_SIZE = 3;

	private int craftingOrientation = 0;
	private final OrderedRecipeInventory recipeInventory = new OrderedRecipeInventory();
	private final ArrayList<Gui> guis = new ArrayList<>();
	public WorkbenchBlockEntity(BlockPos blockPos, BlockState blockState) {
		super(Grafting.WORKBENCH_BLOCK_ENTITY_TYPE, blockPos, blockState, GRID_AREA);
	}

	@Override
	public boolean canPlayerUse(PlayerEntity player) {
		return false;
	}

//	@Override
//	public Text getDisplayName() {
//		// TODO: Localize
//		return Text.of("ASDF??");
//	}
//
//	@Override
//	public @Nullable ScreenHandler createMenu(int syncId, PlayerInventory playerInventory, PlayerEntity player) {
//		return new Gui((ServerPlayerEntity) player)
//				.openAsScreenHandler(syncId, playerInventory, player);
//	}

	public void openMenu(ServerPlayerEntity player) {
		Optional<AttachedInventory> attachedInventory = Optional.empty();

		for(BlockPos p : new BlockPos[]{
				pos.north(1),
				pos.east(1),
				pos.south(1),
				pos.west(1)
		}) {
			assert world != null;
			BlockEntity be = world.getBlockEntity(p);
			if(
					!(be instanceof WorkbenchBlockEntity) && // It gets annoying
					be instanceof Inventory &&
					AttachedInventory.valid((Inventory) be)
			) {
				attachedInventory = Optional.of(new AttachedInventory((Inventory) be));
			}
		}

		new Gui(player, attachedInventory).open();
	}

	@Override
	public void markDirty() {
		for(Gui gui : guis) {
			gui.update();
		}
		super.markDirty();
	}

	@Override
	protected void writeNbt(NbtCompound nbt, RegistryWrapper.WrapperLookup registries) {
		super.writeNbt(nbt, registries);
		nbt.putByte(NBT_ORIENTATION_KEY, (byte) craftingOrientation);
	}

	@Override
	protected void readNbt(NbtCompound nbt, RegistryWrapper.WrapperLookup registries) {
		super.readNbt(nbt, registries);
		craftingOrientation = nbt.getByte(NBT_ORIENTATION_KEY);
	}

	public ItemStack getResult(ServerPlayerEntity player) {
		World world = player.getWorld();
		CraftingRecipeInput craftingRecipeInput = recipeInventory.createRecipeInput();
		return CrafterBlock.getCraftingRecipe(world, craftingRecipeInput)
				.map(recipe -> recipe.value()
						.craft(craftingRecipeInput, world.getRegistryManager())
				)
				.orElse(ItemStack.EMPTY);
	}

	public void rotate(int amount) {
		craftingOrientation = PlacementUtils.clampRotation(craftingOrientation + amount);
		markDirty();
	}

	public class OrderedRecipeInventory implements RecipeInputInventory {
		public int[] getRotatedPlacementIndexes(int slot) {
			return PlacementUtils.asIndexes(
					PlacementUtils.rotate(
							PlacementUtils.getPlacements(WorkbenchBlockEntity.this.getStack(slot), (ServerWorld) world),
							craftingOrientation
					)
			);
		}

		@Override
		public List<ItemStack> getHeldStacks() {
			DefaultedList<ItemStack> stacks = DefaultedList.ofSize(size(), ItemStack.EMPTY);

			for(int i = 0; i < filledSlots(); i++) {
				int[] placements = getRotatedPlacementIndexes(i);

				for (int slot : placements) {
					if(stacks.get(slot).isEmpty()) {
						stacks.set(slot, WorkbenchBlockEntity.this.getStack(i));
						break;
					}
				}
			}

			return stacks;
		}

		private int getOrderedSlot(int positionalSlot) {
			boolean[] filledSlotMatrix = new boolean[size()];
			Arrays.fill(filledSlotMatrix, false);

			for(int i = 0; i <= filledSlots(); i++) {
				int[] placements = getRotatedPlacementIndexes(i);

				for (int slot : placements) {
					if (slot == positionalSlot) {
						return i;
					}

					if (!filledSlotMatrix[slot]) {
						filledSlotMatrix[slot] = true;
						break;
					}
				}
			}

			return -1;
		}

		@Override
		public int getWidth() {
			return GRID_SIZE;
		}

		@Override
		public int getHeight() {
			return GRID_SIZE;
		}

		@Override
		public int size() {
			return WorkbenchBlockEntity.this.size();
		}

		@Override
		public boolean isEmpty() {
			return WorkbenchBlockEntity.this.isEmpty();
		}

		@Override
		public ItemStack getStack(int slot) {
			return WorkbenchBlockEntity.this.getStack(getOrderedSlot(slot));
		}

		@Override
		public ItemStack removeStack(int slot, int amount) {
			return WorkbenchBlockEntity.this.removeStack(getOrderedSlot(slot), amount);
		}

		@Override
		public ItemStack removeStack(int slot) {
			return WorkbenchBlockEntity.this.removeStack(getOrderedSlot(slot));
		}

		@Override
		public void setStack(int slot, ItemStack stack) {
			int orderedSlot = getOrderedSlot(slot);
			if(orderedSlot == -1) {
				pushStack(stack);
			} else {
				WorkbenchBlockEntity.this.setStack(orderedSlot, stack);
			}
		}

		@Override
		public void markDirty() {
			WorkbenchBlockEntity.this.markDirty();
		}

		@Override
		public boolean canPlayerUse(PlayerEntity player) {
			return WorkbenchBlockEntity.this.canPlayerUse(player);
		}

		@Override
		public void provideRecipeInputs(RecipeMatcher finder) {
			for(ItemStack itemStack : this.getHeldStacks()) {
				finder.addUnenchantedInput(itemStack);
			}
		}

		@Override
		public void clear() {
			WorkbenchBlockEntity.this.clear();
		}
	}

	public class Gui extends CursorListeningSimpleGui {
		private static final Identifier GUI_FONT_ID = Identifier.of(Grafting.MOD_ID, "workbench_gui");

		private static final Identifier SLOT_ORDER_OVERLAY_FONT_ID = Identifier.of(Grafting.MOD_ID, "slot_order_overlay");
		private static final char[][] SLOT_ORDER_OVERLAY_FONT_MAP = {
				{ '0', '1', '2', '3', '4', '5', '6', '7' },
				{ '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' },
				{ 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n' }
		};

		private static final int BUTTON_COLUMN = 1;
		private static final int GRID_START_COLUMN = BUTTON_COLUMN + 1;
		private static final int RESULT_COLUMN = GRID_START_COLUMN + GRID_SIZE + 2;
		private static final int BASE_HEIGHT = 3;

		private final CraftingResultInventory resultInventory = new CraftingResultInventory();
		private AttachedInventory attachedInventory;
		private ItemStack slotOrderPreviewStack = ItemStack.EMPTY;
		public Gui(ServerPlayerEntity player) {
			this(player, Optional.empty());
		}

		public Gui(ServerPlayerEntity player, Optional<AttachedInventory> attachedInventory) {
			super(
					attachedInventory.isEmpty() ?
							ScreenHandlerType.GENERIC_9X3 :
							attachedInventory.get().extendedScreenHandlerType(BASE_HEIGHT),
					player,
					false
			);
			attachedInventory.ifPresent(inventory -> this.attachedInventory = inventory);
			guis.add(this);

			setSlot(BUTTON_COLUMN, new RotateGridButton(Icons.ROTATE_CLOCKWISE,1));
			setSlot(BUTTON_COLUMN + 9, new RotateGridButton(Icons.ROTATE_COUNTER_CLOCKWISE, -1));
			setSlotRedirect(
					RESULT_COLUMN + 9,
					new CraftingResultGridSlot(player, recipeInventory, resultInventory, 0, 0, 0)
			);
			for(int i = 0; i < GRID_AREA; i++) {
				int row = (i / GRID_SIZE);
				setSlotRedirect(
						GRID_START_COLUMN + (i % GRID_SIZE) + (row * 9),
						new GridSlot(i)
				);
			}

			attachedInventory.ifPresent(inventory -> {
				inventory.redirectSlots(BASE_HEIGHT, this);
			});

			update();
		}

		public int getUnextendedSize() {
			return BASE_HEIGHT * 9;
		}

		protected void update() {
			resultInventory.setStack(0, getResult(player));
			updateTitle();
		}

		@Override
		public void close() {
			guis.remove(this);
			super.close();
		}

		@Override
		public void onCursorStackChange(ItemStack stack) {
			var different = stack.isEmpty() || !ItemStack.areItemsAndComponentsEqual(stack, slotOrderPreviewStack);
			this.slotOrderPreviewStack = stack;
			if(different) {
				updateTitle();
			}
		}

		private void updateTitle() {
			var title = Text.empty();

			title.append(
					Text.literal(attachedInventory != null ? "-1." : "-0.")
							.setStyle(Style.EMPTY.withFont(GUI_FONT_ID).withColor(Formatting.WHITE))
			);

			title.append(
					Text.literal(getSlotOrderLiteral())
							.setStyle(Style.EMPTY.withFont(SLOT_ORDER_OVERLAY_FONT_ID).withColor(Formatting.WHITE))
			);

			title.append(Text.translatable("container.crafting"));
			setTitle(title);
		}

		private String getSlotOrderLiteral() {
			StringBuilder out = new StringBuilder();

			out.append(" ".repeat(GRID_START_COLUMN));

			if(slotOrderPreviewStack.isEmpty()) {
				return out.toString();
			}

			var placements = PlacementUtils.rotate(
					PlacementUtils.getPlacements(slotOrderPreviewStack, (ServerWorld) world),
					craftingOrientation
			);

			for(int i = 0; i < 3; i++) {
				boolean columnEmpty = true;
				for(int j = 0; j < 3; j++) {
					int order = placements[j * 3 + i] - Math.max(filledSlots(), 1);

					if(order < 0) {
						continue;
					}

					if(columnEmpty) {
						columnEmpty = false;
						out.append('.');
					} else {
						out.append('-');
					}

					out.append(SLOT_ORDER_OVERLAY_FONT_MAP[j][order]);
				}

				if(columnEmpty) {
					out.append(' ');
				}
			}

			out.append("___");

			return out.toString();
		}

		/*
		* See: CraftingScreenHandler#quickMove
		 */
		@Override
		public ItemStack quickMove(int index) {
			ItemStack returnStack = ItemStack.EMPTY;
			Slot slot = this.getSlotRedirectOrPlayer(index);
			boolean isResultSlot = slot instanceof CraftingResultSlot;

			if(slot != null && slot.hasStack() && !(slot instanceof VirtualSlot)) {
				ItemStack operatingStack;
				if(slot instanceof GridSlot) {
					operatingStack = topStack();
				} else {
					operatingStack = slot.getStack();
				}
				returnStack = operatingStack.copy();

				if(isResultSlot) {
					operatingStack.getItem().onCraftByPlayer(operatingStack, world, player);

					if(!this.insertItem(operatingStack, this.getVirtualSize(), this.getVirtualSize() + 36, true)) {
						return ItemStack.EMPTY;
					}

					slot.onQuickTransfer(operatingStack, returnStack);
				} else if(index < getUnextendedSize()) {
					if(!this.insertItem(operatingStack, this.getUnextendedSize(), this.getVirtualSize() + 36, false)) {
						return ItemStack.EMPTY;
					}
				} else {
					if(similarStacks(operatingStack).isEmpty() && hasRoom()) {
						pushStack(operatingStack.copyAndEmpty());
					} else {
						distributeStack(operatingStack, operatingStack.getCount());
					}
				}

				if (operatingStack.isEmpty()) {
					slot.setStack(ItemStack.EMPTY);
				} else {
					slot.markDirty();
				}

				if(returnStack.getCount() == operatingStack.getCount()) {
					return ItemStack.EMPTY;
				}

				slot.onTakeItem(player, returnStack);
				if(isResultSlot) {
					player.dropItem(returnStack, false);
				}
			} else if (slot instanceof VirtualSlot) {
				return slot.getStack();
			}

			return returnStack;
		}

		//		- Inserting single items changes the shape
		//		- Inserting a full stack will attempt to distribute the stack, or insert if that stack isn't in the workbench
		public class GridSlot extends Slot {
			public GridSlot(int index) {
				super(WorkbenchBlockEntity.this.recipeInventory, index, index, 0);
			}

			@Override
			public ItemStack insertStack(ItemStack stack, int count) {
				if(stack.isEmpty()) {
					return stack;
				}

				int allowedCount = Math.min(count, stack.getCount());
				if(
						(getStack().isEmpty() || similarStacks(stack).isEmpty()) &&
								hasRoom()
				) { // Right click
					pushStack(stack.split(allowedCount));
				} else {
					distributeStack(stack, allowedCount);
					if(stack.isEmpty()) {
						return ItemStack.EMPTY;
					}
				}

				return stack;
			}

			@Override
			public ItemStack takeStack(int amount) {
				var o = super.takeStack(amount);
				if(!getStack().isEmpty()) {
					distributeStack(getStack());
				}
				return o;
			}
		}

		public class CraftingResultGridSlot extends CraftingResultSlot {
			public CraftingResultGridSlot(PlayerEntity player, RecipeInputInventory input, Inventory inventory, int index, int x, int y) {
				super(player, input, inventory, index, x, y);
			}

			@Override
			public void onTakeItem(PlayerEntity player, ItemStack stack) {
				super.onTakeItem(player, stack);
				WorkbenchBlockEntity.this.markDirty(); // Make sure the OrderedContainer gets cleaned so that everything is in its rightful place
			}
		}

		public class RotateGridButton extends GuiElement {
			public RotateGridButton(PolymerModelData model, int bias) {
				super(
						Icons.getStack(model),
						new ItemClickCallback() {
							@Override
							public void click(int i, ClickType clickType, SlotActionType slotActionType) {
								switch(clickType) {
									case ClickType.MOUSE_LEFT -> rotate(bias);
									case ClickType.MOUSE_RIGHT -> rotate(bias * -1);
								}
							}
						}
				);
			}
		}
	}
}
