package net.sys42.dakedres.grafting.data;

import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import com.mojang.serialization.DynamicOps;
import com.mojang.serialization.codecs.PrimitiveCodec;
import net.minecraft.util.Identifier;
import net.sys42.dakedres.grafting.Grafting;

import java.util.stream.IntStream;

public class PlacementOrder {
	public static final PlacementOrder DEFAULT_FALLBACK = new PlacementOrder(new int[]{
			0,1,2,
			3,4,5,
			6,7,8
	});
	public static final Identifier DEFAULT_ID = Identifier.of(Grafting.MOD_ID, "default");

	int[] orderMatrix;
	public PlacementOrder(int[] orderMatrix) {
		this.orderMatrix = orderMatrix;
	}

	public int[] getOrderMatrix() {
		return orderMatrix;
	}

	public static final Codec<PlacementOrder> CODEC = new PrimitiveCodec<>() {
		@Override
		public <T> DataResult<PlacementOrder> read(DynamicOps<T> ops, T input) {
			return ops.getIntStream(input).flatMap(intStream -> {
				var om = intStream.toArray();
				try {
					PlacementOrder.validate(om);
				} catch (Exception e) {
					return DataResult.error(e::getMessage);
				}

				return DataResult.success(new PlacementOrder(om));
			});
		}

		@Override
		public <T> T write(DynamicOps<T> ops, PlacementOrder value) {
			return ops.createIntList(IntStream.of(value.getOrderMatrix()));
		}

		@Override
		public String toString() {
			return "PlacementOrder";
		}
	};

	public static void validate(int[] orderMatrix) throws Exception {
		if(orderMatrix.length != 9) {
			throw new Exception("Not a complete matrix. Expected an array of at least 9 integers between 0 and 8.");
		}

		for(int i = 0; i < 9; i++) {
			if(orderMatrix[i] >= 9 || orderMatrix[i] < 0) {
				throw new Exception(String.format("At index {}, expected integer between 8 and 0", i));
			}
		}
	}
}
