package net.sys42.dakedres.grafting.data;

import com.google.gson.*;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.minecraft.registry.Registry;
import net.minecraft.resource.Resource;
import net.minecraft.util.Identifier;
import net.sys42.dakedres.grafting.Grafting;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Map;

public class PlacementOrderReloadListener {
	static public void init() {
		ServerLifecycleEvents.START_DATA_PACK_RELOAD.register((server, resourceManager) -> {
			// Thank you tomalbrc! https://github.com/tomalbrc/filament/blob/main/src/main/java/de/tomalbrc/filament/util/FilamentSynchronousResourceReloadListener.java#L70

			var resources = resourceManager.findResources(
					"grafting/placement_order",
					path -> path.getPath().endsWith(".json")
			);

			for(Map.Entry<Identifier, Resource> entry : resources.entrySet()) {
				load(entry.getKey(), entry.getValue());
			}

//			Grafting.LOGGER.info("Loaded {} placement orders.", Grafting.PLACEMENT_ORDER_REGISTRY.getEntrySet().size());
		});
	}

	static private void load(Identifier id, Resource resource) {
		JsonObject json;

		try(BufferedReader reader = resource.getReader()) {
			load(id, reader);
		} catch(IOException | IllegalStateException e) {
			error(id, e);
			return;
		}
	}

	static private void load(Identifier id, BufferedReader reader) {
		Gson gson = new GsonBuilder().create();
		JsonElement rootElement = JsonParser.parseReader(reader);

		try {
			int[] orderMatrix = gson.fromJson(rootElement, int[].class);

			PlacementOrder.validate(orderMatrix);
		} catch(Exception e) {
			error(id, e);
		}

//		Grafting.PLACEMENT_ORDER_REGISTRY
//		PlacementOrder.Data = gson.fromJson(rootElement, PlacementOrder.Data.class);
	}

	static private void error(Identifier id, Exception e) {
		Grafting.LOGGER.error("Could not load placement order \"{}\": {}", id, e);
	}
}
