package net.sys42.dakedres.grafting.ui;

import eu.pb4.sgui.api.elements.GuiElementInterface;
import eu.pb4.sgui.api.gui.SimpleGui;
import eu.pb4.sgui.api.gui.SlotGuiInterface;
import eu.pb4.sgui.virtual.SguiScreenHandlerFactory;
import eu.pb4.sgui.virtual.inventory.VirtualScreenHandler;
import eu.pb4.sgui.virtual.inventory.VirtualSlot;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.network.packet.s2c.play.OpenScreenS2CPacket;
import net.minecraft.screen.NamedScreenHandlerFactory;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.screen.ScreenHandlerType;
import net.minecraft.screen.slot.Slot;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import org.jetbrains.annotations.Nullable;

import java.util.OptionalInt;

public abstract class CursorListeningSimpleGui extends SimpleGui {
	private CursorListeningScreenHandler screenHandler = null;
	private Text title;

	public CursorListeningSimpleGui(ScreenHandlerType<?> type, ServerPlayerEntity player, boolean manipulatePlayerSlots) {
		super(type, player, manipulatePlayerSlots);
	}

	@Override
	public void setSlot(int index, GuiElementInterface element) {
		super.setSlot(index, element);
		if (this.isOpen() && this.autoUpdate) {
			this.screenHandler.setSlot(index, new VirtualSlot(this, index, 0, 0));
		}

	}

	@Override
	public void setSlotRedirect(int index, Slot slot) {
		super.setSlotRedirect(index, slot);
		if (this.isOpen() && this.autoUpdate) {
			this.screenHandler.setSlot(index, slot);
		}

	}

	@Override
	public void clearSlot(int index) {
		super.clearSlot(index);
		this.hasRedirects = true;
		if (this.isOpen() && this.autoUpdate) {
			this.screenHandler.setSlot(index, new VirtualSlot(this, index, 0, 0));
		}

	}

	@Override
	public boolean isOpen() {
		return this.screenHandler != null && this.screenHandler == this.player.currentScreenHandler;
	}

	public Text getTitle() {
		return this.title;
	}
	
	@Override
	public void setTitle(Text title) {
		this.title = title;
		if (this.isOpen()) {
			this.player.networkHandler.sendPacket(new OpenScreenS2CPacket(this.syncId, this.type, title));
			this.screenHandler.syncState();
		}

	}

	@Override
	protected boolean sendGui() {
		this.reOpen = true;
		OptionalInt temp = this.player.openHandledScreen(getFactory());
		this.reOpen = false;
		if (temp.isPresent()) {
			this.syncId = temp.getAsInt();
			if (this.player.currentScreenHandler instanceof CursorListeningScreenHandler) {
				this.screenHandler = (CursorListeningScreenHandler) this.player.currentScreenHandler;
				return true;
			}
		}

		return false;
	}

	private NamedScreenHandlerFactory getFactory() {
		return new SguiScreenHandlerFactory<CursorListeningSimpleGui>(
				this,
				(syncId, inv, player) -> new CursorListeningScreenHandler(getType(), syncId, this, player)
		);
	}

	@Override
	public ScreenHandler openAsScreenHandler(int syncId, PlayerInventory playerInventory, PlayerEntity player) {
		if (!this.player.isDisconnected() && player == this.player && !this.isOpen()) {
			this.beforeOpen();
			this.onOpen();
			this.screenHandler = new CursorListeningScreenHandler(this.getType(), syncId, this, player);
			return this.screenHandler;
		} else {
			return null;
		}
	}

	public abstract void onCursorStackChange(ItemStack stack);

	@Override
	public void close(boolean screenHandlerIsClosed) {
		if ((this.isOpen() || screenHandlerIsClosed) && !this.reOpen) {
			if (!screenHandlerIsClosed && this.player.currentScreenHandler == this.screenHandler) {
				this.player.closeHandledScreen();
				this.screenHandler = null;
			}

			this.player.currentScreenHandler.syncState();
			this.onClose();
		} else {
			this.reOpen = false;
		}

	}

	public class CursorListeningScreenHandler extends VirtualScreenHandler {
		public CursorListeningScreenHandler(@Nullable ScreenHandlerType<?> type, int syncId, SlotGuiInterface gui, PlayerEntity player) {
			super(type, syncId, gui, player);
		}

		@Override
		public void setCursorStack(ItemStack stack) {
			super.setCursorStack(stack);
			onCursorStackChange(stack);
		}
	}
}
